'===============================================================
' Program:  XMLTV to TVGuide for PPC data converter
' Version:  0.8
'
' Author:   Alex Egorov [alxeg@yandex.ru]
' Copyright (c) 2004-2007 Alex Egorov. All rights reserved.
'===============================================================

If Wscript.Arguments.Count >= 1 Then
    xmlName = WScript.Arguments(0)
    set fso = CreateObject("Scripting.FileSystemObject")

    'if Wscript.Arguments.Count >= 2 then
    '    outputFolder = fso.GetAbsolutePathName(WScript.Arguments(1))
    '    if not fso.FolderExists(outputFolder) then fso.CreateFolder(outputFolder)
    'end if

    On Error Resume Next
    Set xml = CreateObject("Msxml2.DOMDocument")

    if xml is nothing then
        MsgBox "       You have no MSXML installed        " & Chr(13) & _
               "Download it from http://www.microsoft.com/ ", vbCritical

        WScript.Quit
    end if
    On Error Goto 0

    outputFolder = "_tvguide_data_"
    if not fso.FolderExists(outputFolder) then fso.CreateFolder(outputFolder)

    'xml.resolveExternals = false

    if not xml.load(xmlName) then
        MsgBox "Can't load file """ & xmlName & """ " & Chr(13) & _
                xml.parseError.reason & " at line " & xml.parseError.line, vbCritical
        WScript.Quit
    end if

    set root = xml.documentElement
    Set channelsNodes = root.selectNodes("/tv/channel")

    if channelsNodes.length=0 then
        MsgBox "Can't find channels", vbCritical
        WScript.Quit
    end if

    For Each channel In channelsNodes
        chId = channel.selectSingleNode("./@id").nodeValue

        chName = ""
        set chNameNode = channel.selectSingleNode("./display-name/text()")
        if not chNameNode is nothing then chName = chNameNode.nodeValue
        if chName="" then chName=chId

        'WScript.Echo chName

        set newXml = CreateObject("Msxml2.DOMDocument")
        newXml.appendChild(newXml.createProcessingInstruction("xml", "version=""1.0"" encoding=""UTF-8"""))

        set newRoot = newXml.createElement("site")
        newRoot.setAttribute "name", "xmltv"
        newXml.appendChild(newRoot)
        set chElem = newXml.createElement("channel")
        chElem.setAttribute "name", chName
        newRoot.appendChild chElem

        set progNodes = root.selectNodes("/tv/programme[@channel="""&chId&"""]")

        For Each prog In progNodes
            dateStart = ParseDate(prog.selectSingleNode("./@start").nodeValue)

            set stopAttr = prog.selectSingleNode("./@stop")
            if not stopAttr is nothing then
                dateStop = ParseDate(stopAttr.nodeValue)
                duration = DateDiff("n", dateStart, dateStop)
            else
                set dateStop = nothing
                duration = 0
            end if

            title = ""
            title = prog.selectSingleNode("./title/text()").nodeValue

            sTitle = ""
            set sTitleNode = prog.selectSingleNode("./sub-title/text()")
            if not sTitleNode is nothing then sTitle = sTitleNode.nodeValue

            desc = ""
            set descNode = prog.selectSingleNode("./desc/text()")
            if not descNode is nothing then desc = descNode.nodeValue

            category = ""
            set catNode = prog.selectSingleNode("./category/text()")
            if not catNode is nothing then category = catNode.nodeValue

            set progElem = newXml.createElement("program")
            progElem.setAttribute "time", ConvertDate(dateStart)
            progElem.setAttribute "length", duration
            set nameElem = newXml.createElement("name")
            nameElem.text = title
            if sTitle<>"" then nameElem.text = nameElem.text & ": " & sTitle
            if category<>"" then nameElem.text = nameElem.text & " [" & category & "] "

            progElem.appendChild nameElem

            if desc<>"" then
                set descElem = newXml.createElement("announcement")
                descElem.text = desc
                progElem.appendChild descElem
            end if
            chElem.appendChild progElem
        next

        fileOutputName = chName
        fileOutputName = Replace(fileOutputName, ":", " ")
        fileOutputName = Replace(fileOutputName, "/", " ")
        fileOutputName = Replace(fileOutputName, "\", " ")
        newXml.save(outputFolder&"\"&fileOutputName&".xml")

    Next

else
    MsgBox "XMLTV to TVGuide for PPC data converter" & Chr(13) & _
           "by Alex Egorov [alxeg@yandex.ru]" & Chr(13)  & Chr(13) & _
           "Usage: " & WScript.ScriptName & " <input xml file>", vbInformation
    WScript.Quit
end if

function ParseDate(strValue)
    set re = new RegExp
    re.Pattern = "(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})?(\s+([\+-]\d+))?"
    'value = re.Replace(strValue, "$1-$2-$3 $4:$5")
    set matches = re.Execute(strValue)

    if matches.count<>0 then
        set submatches = matches(0).SubMatches
        value = DateSerial(submatches(0), submatches(1), submatches(2)) + TimeSerial(submatches(3), submatches(4), submatches(5))
        value = DateAdd("h", -submatches(7)/100, value)
        ParseDate = value
    else
        ParseDate = ""
    end if

end function

function ConvertDate(value)
  if Month(value) <10 then m  = "0" & Month(value)  else m  = Month(value)
  if Day(value)   <10 then d  = "0" & Day(value)    else d  = Day(value)
  if Hour(value)  <10 then h  = "0" & Hour(value)   else h  = Hour(value)
  if Minute(value)<10 then mn = "0" & Minute(value) else mn = Minute(value)
  ConvertDate = Year(value) & "-" & m & "-" & d & " " &  h & ":" & mn & " GMT"
  'ConvertDate = Format(Year(value), "0000") & "-" & Format(Month(value),"00") & "-" & Format(Day(value),"00") & " " & _
  '              Format(Hour(value),"00") & ":" & Format(Minute(value),"00") & " GMT"
end function
